﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Infragistics.MultiTouch.Gestures
{
    public class Gesture : List<Direction>, IEquatable<Gesture>
	{
		#region Constructor
		/// <summary>
		/// Initializes a new instance of the <see cref="Gesture"/> class.
		/// </summary>
		public Gesture()
        {
        }
		#endregion Constructor

		#region Operator Impementations
		/// <summary>
		/// Implements the operator ==.
		/// </summary>
		/// <param name="g1">The g1.</param>
		/// <param name="g2">The g2.</param>
		/// <returns>The result of the operator.</returns>
        public static bool operator ==(Gesture g1, Gesture g2)
        {
            Gesture g = g1 ?? g2;

            if (object.ReferenceEquals(g, null))
            {
                return true;
            }

            if (object.ReferenceEquals(g1, null) || object.ReferenceEquals(g2, null))
            { 
                return false; 
            }

            return g1.Equals(g2);
        }

		/// <summary>
		/// Implements the operator !=.
		/// </summary>
		/// <param name="g1">The g1.</param>
		/// <param name="g2">The g2.</param>
		/// <returns>The result of the operator.</returns>
        public static bool operator !=(Gesture g1, Gesture g2)
        {
            return !(g1 == g2);
        }
		#endregion Operator Impementations

		#region Static Methods
		/// <summary>
        /// Parses a string command representation to a gesture object.
        /// </summary>
        /// <param name="value"></param>
        /// <returns></returns>
        public static Gesture Parse(string value)
        {
            if (value == null) return null;

            Gesture gResult = new Gesture();

            foreach (char c in value.ToCharArray())
            {
                switch (c.ToString().ToUpper())
                {
                    case "L":
                        gResult.Add(Direction.Left);
                        break;
                    case "U":
                        gResult.Add(Direction.Up);
                        break;
                    case "D":
                        gResult.Add(Direction.Down);
                        break;
                    case "R":
                        gResult.Add(Direction.Right);
                        break;
                    default:
                        throw new ArgumentException(string.Format(
                            "Unknown character value '{0}'. Could not convert it to a specific gesture direction.", c.ToString()));
                }
            }

            return gResult;
        }
		#endregion Static Methods

		#region IEquatable<Gesture> Members
		/// <summary>
		/// Equalses the specified other.
		/// </summary>
		/// <param name="other">The other.</param>
		/// <returns></returns>
		public bool Equals(Gesture other)
        {
            if (other == null || Count != other.Count) return false;

            for (int i = 0; i < Count; i++)
            {
                if (this[i] != other[i])
                {
					return false;
                }
            }

            return true;
        }
        #endregion

		#region Override Methods
		/// <summary>
		/// Returns a hash code for this instance.
		/// </summary>
		/// <returns>
		/// A hash code for this instance, suitable for use in hashing algorithms and data structures like a hash table. 
		/// </returns>
        public override int GetHashCode()
        {
            return ToString().GetHashCode();
        }

		/// <summary>
		/// Determines whether the specified <see cref="System.Object"/> is equal to this instance.
		/// </summary>
		/// <param name="obj">The <see cref="System.Object"/> to compare with this instance.</param>
		/// <returns>
		/// 	<c>true</c> if the specified <see cref="System.Object"/> is equal to this instance; otherwise, <c>false</c>.
		/// </returns>
        public override bool Equals(object obj)
        {
            if (obj == null || !(obj is Gesture))
            {
                return false;
            }

            return Equals((Gesture)obj);
        }

		/// <summary>
		/// Returns a <see cref="System.String"/> that represents this instance.
		/// </summary>
		/// <returns>
		/// A <see cref="System.String"/> that represents this instance.
		/// </returns>
        public override string ToString()
        {
            return ToString(string.Empty);
        }

		/// <summary>
		/// Returns a <see cref="System.String"/> that represents this instance.
		/// </summary>
		/// <param name="delimiter">The delimiter.</param>
		/// <returns>
		/// A <see cref="System.String"/> that represents this instance.
		/// </returns>
        public string ToString(string delimiter)
        {
            StringBuilder result = new StringBuilder();

            foreach (Direction dir in this)
            {
                if (result.Length > 0)
                {
                    result.Append(delimiter);
                }

                result.Append(dir.ToString().Substring(0, 1));
            }

            return result.ToString();
        }
		#endregion Override Methods
	}
}
