﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using Infragistics.MultiTouch.Gestures;

namespace Infragistics.MultiTouch.Gestures.Controls
{
    public delegate void GestureRoutedEventHandler(object sender, GestureRoutedEventArgs e);

    public class GestureRoutedEventArgs : RoutedEventArgs
	{
		#region Constructors
		/// <summary>
		/// Initializes a new instance of the <see cref="GestureRoutedEventArgs"/> class.
		/// </summary>
		/// <param name="gesture">The gesture.</param>
		public GestureRoutedEventArgs(Gesture gesture) : base()
        {
            this.Gesture = gesture;
        }

		/// <summary>
		/// Initializes a new instance of the <see cref="GestureRoutedEventArgs"/> class.
		/// </summary>
		/// <param name="source">The source.</param>
		/// <param name="gesture">The gesture.</param>
        public GestureRoutedEventArgs(object source, Gesture gesture) : this(gesture)
        {
            this.OriginalSource = source;
        }
		#endregion Constructors

		#region Public Properties
		/// <summary>
		/// Gets or sets the gesture.
		/// </summary>
		/// <value>The gesture.</value>
		public Gesture Gesture { get; private set; }
		/// <summary>
		/// Gets a reference to the object that raised the event.
		/// </summary>
		/// <value></value>
		/// <returns>The object that raised the event.</returns>
        public new object OriginalSource { get; private set; }
		#endregion Public Properties
	}
}
