﻿using System;
using System.Windows;
using System.Windows.Interactivity;
using System.Windows.Controls;
using System.Windows.Media;

namespace Infragistics.Silverlight.Applications.Stocks.Behaviors
{
	public class VisibilityToggleTrigger : TriggerAction<FrameworkElement>
	{
		#region Dependency Properties
		public static DependencyProperty ElemenetNameProperty = DependencyProperty.Register("ElementName", typeof(string), typeof(VisibilityToggleTrigger), null);
		public string ElementName { get { return (string)GetValue(ElemenetNameProperty); } set { SetValue(ElemenetNameProperty, value); } }

		public static DependencyProperty SearchScopeProperty = DependencyProperty.Register("SearchScope", typeof(ElementScope), typeof(VisibilityToggleTrigger), new PropertyMetadata(ElementScope.Application));
		public ElementScope SearchScope { get { return (ElementScope)GetValue(SearchScopeProperty); } set { SetValue(SearchScopeProperty, value); } }
		#endregion Dependency Properties

		#region Protected Overrides
		/// <summary>
		/// Called when [attached].
		/// </summary>
		protected override void OnAttached()
		{
			base.OnAttached();
		}

		/// <summary>
		/// Called when [detaching].
		/// </summary>
		protected override void OnDetaching()
		{
			base.OnDetaching();
		}

		/// <summary>
		/// Invokes the specified parameter.
		/// </summary>
		/// <param name="parameter">The parameter.</param>
		protected override void Invoke(object parameter)
		{
			FrameworkElement rootElement = null;

			switch (SearchScope)
			{
				case ElementScope.Application:
					rootElement = (FrameworkElement) Application.Current.RootVisual;
					break;
				case ElementScope.Parent:
					rootElement = (FrameworkElement) AssociatedObject.Parent;
					break;
				case ElementScope.Child:
					rootElement = (FrameworkElement) AssociatedObject;
					break;
			}

			if (rootElement != null)
			{
				var fe = rootElement.FindVisualChildByName<FrameworkElement>(ElementName);
				
				if (fe != null)
				{
					fe.Visibility = fe.Visibility == Visibility.Visible ? Visibility.Collapsed : Visibility.Visible;
				}
			}
		}
		#endregion Protected Overrides
	}

	public enum ElementScope
	{
		Application,
		Parent,
		Child
	}
}
