﻿using System;
using System.Windows.Data;
using System.Windows;

namespace Infragistics.Silverlight.Applications.Stocks.Converters
{
	public class BoolToVisibilityValueConverter : DependencyObject, IValueConverter
	{
		#region Dependency Properties
		public static DependencyProperty TrueValueProperty = DependencyProperty.Register("TrueValue", typeof(Visibility), typeof(BoolToVisibilityValueConverter), new PropertyMetadata(Visibility.Visible));
		public Visibility TrueValue { get { return (Visibility)GetValue(TrueValueProperty); } set { SetValue(TrueValueProperty, value); } }

		public static DependencyProperty FalseValueProperty = DependencyProperty.Register("FalseValue", typeof(Visibility), typeof(BoolToVisibilityValueConverter), new PropertyMetadata(Visibility.Collapsed));
		public Visibility FalseValue { get { return (Visibility)GetValue(FalseValueProperty); } set { SetValue(FalseValueProperty, value); } }
		#endregion Dependency Properties

		/// <summary>
		/// Modifies the source data before passing it to the target for display in the UI.
		/// </summary>
		/// <param name="value">The source data being passed to the target.</param>
		/// <param name="targetType">The <see cref="T:System.Type"/> of data expected by the target dependency property.</param>
		/// <param name="parameter">An optional parameter to be used in the converter logic.</param>
		/// <param name="culture">The culture of the conversion.</param>
		/// <returns>
		/// The value to be passed to the target dependency property.
		/// </returns>
		public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			if (value == null) return TrueValue;

			return ((bool)value) ? TrueValue : FalseValue;
		}

		/// <summary>
		/// Modifies the target data before passing it to the source object.  This method is called only in <see cref="F:System.Windows.Data.BindingMode.TwoWay"/> bindings.
		/// </summary>
		/// <param name="value">The target data being passed to the source.</param>
		/// <param name="targetType">The <see cref="T:System.Type"/> of data expected by the source object.</param>
		/// <param name="parameter">An optional parameter to be used in the converter logic.</param>
		/// <param name="culture">The culture of the conversion.</param>
		/// <returns>
		/// The value to be passed to the source object.
		/// </returns>
		public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
		{
			if (value == null) return false;

			return ((Visibility)value) == TrueValue ? true : false;
		}
	}
}
