﻿using System;
using System.Windows;
using System.Windows.Media;
using System.Windows.Controls;
using System.Linq;

namespace Infragistics.Silverlight.Applications.Stocks
{
	public static class VisualTreeExtensions
	{
		public static T FindVisualChildByName<T>(this DependencyObject parent, string name) where T : DependencyObject
		{
			for (int i = 0; i < VisualTreeHelper.GetChildrenCount(parent); i++)
			{
				var child = VisualTreeHelper.GetChild(parent, i);
				string controlName = child.GetValue(Control.NameProperty) as string;
				if (controlName == name)
				{
					return child as T;
				}
				else
				{
					T result = FindVisualChildByName<T>(child, name);
					if (result != null)
					{
						return result;
					}
				}
			}

			return null;
		}

		/// <summary>
		/// Finds the first parent of type T of the current object.
		/// </summary>
		/// <typeparam name="T"></typeparam>
		/// <param name="obj">The obj.</param>
		/// <returns></returns>
		public static T FindParentByType<T>(this DependencyObject obj) where T : DependencyObject
		{
			DependencyObject p = obj;
			
			while((p = VisualTreeHelper.GetParent(p)) != null)
			{
				if (p is T) return p as T;
			}

			return null;
		}
	}
}
