﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Silverlight.Applications.Stocks.ViewModels;
using Microsoft.Phone.Shell;

namespace Infragistics.Silverlight.Applications.Stocks
{
    public partial class App : Application
	{
		#region Constructor
		/// <summary>
		/// Initializes a new instance of the <see cref="App"/> class.
		/// </summary>
        public App()
        {
			Startup += new StartupEventHandler(Application_Startup);
			Exit += new EventHandler(Application_Exit);
            UnhandledException += new EventHandler<ApplicationUnhandledExceptionEventArgs>(Application_UnhandledException);

            InitializeComponent();
        }
		#endregion Constructor

		#region Public Properties
		public StockViewModel ViewModel { get; set; }
		#endregion Public Properties

		#region Private Event Handlers
		/// <summary>
		/// Handles the Exit event of the Application control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
		void Application_Exit(object sender, EventArgs e)
		{
			if (ViewModel != null)
			{
				ViewModel.Save();
			}
		}

		/// <summary>
		/// Handles the Startup event of the Application control.
		/// </summary>
		/// <param name="sender">The source of the event.</param>
		/// <param name="e">The <see cref="System.Windows.StartupEventArgs"/> instance containing the event data.</param>
		void Application_Startup(object sender, StartupEventArgs e)
		{
			
		}

        // Code to execute on Unhandled Exceptions
        private void Application_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // An unhandled exception has occurred, break in the debugger
                System.Diagnostics.Debugger.Break();
            }
            else
            {
                // By default show the error
                e.Handled = true;
                MessageBox.Show(string.Format("{0}{1}{2}", e.ExceptionObject.Message, Environment.NewLine, e.ExceptionObject.StackTrace),
                    			"Error", MessageBoxButton.OK);
            }
        }

        // Code to execute when the application is launching (eg, from Start)
        // This code will not execute when the application is reactivated
        private void Application_Launching(object sender, LaunchingEventArgs e)
        {
        }

        // Code to execute when the application is activated (brought to foreground)
        // This code will not execute when the application is first launched
        private void Application_Activated(object sender, ActivatedEventArgs e)
        {
        }

        // Code to execute when the application is deactivated (sent to background)
        // This code will not execute when the application is closing
        private void Application_Deactivated(object sender, DeactivatedEventArgs e)
        {
        }

        // Code to execute when the application is closing (eg, user hit Back)
        // This code will not execute when the application is deactivated
        private void Application_Closing(object sender, ClosingEventArgs e)
        {
        }

		#endregion Private Event Handlers
	}
}