﻿using System;
using System.ComponentModel;
using System.Net.NetworkInformation;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using Infragistics.Controls.Interactions;
using Infragistics.MultiTouch.Gestures.Controls;
using Microsoft.Phone.Controls;
using System.Windows.Threading;
using Infragistics.Silverlight.Applications.Stocks.ViewModels;
using Microsoft.Phone.Net.NetworkInformation;
using Microsoft.Phone.Shell;
using NetworkInterface = Microsoft.Phone.Net.NetworkInformation.NetworkInterface;

namespace Infragistics.Silverlight.Applications.Stocks
{
    public partial class MainPage : PhoneApplicationPage
    {
        #region Private Member Variables
        private DispatcherTimer _timer;
        private GestureProcessor _gestureTranslator;
        private bool _storyBoardFlag = true;
        private int _storyBoardCount = 0;

        private StockViewModel _vm;
        private int _counter = 0;
        private string _resumeSymbol;
        private bool _showSearchView;
        private string _searchText;
        private PageOrientation _lastOrientation = PageOrientation.Portrait;
        #endregion Private Member Variables

        #region Constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="MainPage"/> class.
        /// </summary>
        public MainPage()
        {
            InitializeComponent();

            SupportedOrientations = SupportedPageOrientation.Portrait | SupportedPageOrientation.Landscape;

            // Setup the basic Gestures and their call backs
            _gestureTranslator = new GestureProcessor(LayoutRoot);

            //_gestureTranslator.GestureHandlers.Add(Infragistics.MultiTouch.Gestures.Gesture.Parse("D"), Gesture_Down);
            //_gestureTranslator.GestureHandlers.Add(Infragistics.MultiTouch.Gestures.Gesture.Parse("U"), Gesture_Up);
            //_gestureTranslator.GestureHandlers.Add(Infragistics.MultiTouch.Gestures.Gesture.Parse("L"), Gesture_Left);
            //_gestureTranslator.GestureHandlers.Add(Infragistics.MultiTouch.Gestures.Gesture.Parse("R"), Gesture_Right);

            viewPortrait.SearchBar.MouseLeftButtonUp += SearchBarMouseLeftButtonUp;
            viewLandScape.SearchBar.MouseLeftButtonUp += SearchBarMouseLeftButtonUp;

            BackKeyPress += MainPageBackKeyPress;
        }

        #endregion Constructor

        #region Event Handlers
        /// <summary>
        /// Handles the Loaded event of the PhoneApplicationPage control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.RoutedEventArgs"/> instance containing the event data.</param>
        private void PhoneApplicationPage_Loaded(object sender, RoutedEventArgs e)
        {
            viewPortrait.FadeIn.Completed += PortraitFadeInCompleted;
            viewLandScape.FadeIn.Completed += LandscapeFadeInCompleted;

            if (IsOrientationLandscape(Orientation))
            {
                SplashImageLandscape.Visibility = System.Windows.Visibility.Visible;
            }

            // Lets grab the data context
            _vm = LayoutRoot.DataContext as StockViewModel;

            ((App)Application.Current).ViewModel = _vm;

            // If we have any saved information for last run, lets load it
            _vm.Load();
            if (_resumeSymbol != null)
            {
                _vm.SelectStockCommand.Execute(_resumeSymbol);
            }

            if (_searchText != null)
            {
                viewSearch.SearchBox.Text = _searchText;
            }
            //_vm.StockSearchFilter = viewSearch.SearchBox.Text;


#if DEBUG
            // Add some default data
            //_vm.AddSymbolToWatchList("MSFT");
            //_vm.AddSymbolToWatchList("GOOG");

            // Select the First Stock
            //_vm.SelectStockCommand.Execute("MSFT");
#endif

            // Lets make sure all of the user controls are using the correct data context
            viewPortrait.DataContext = _vm;
            viewLandScape.DataContext = _vm;
            viewSearch.DataContext = _vm;

            viewPortrait.LayoutRoot.DataContext = _vm;
            viewLandScape.LayoutRoot.DataContext = _vm;
            viewSearch.LayoutRoot.DataContext = _vm;

            NetworkInterfaceType nit = NetworkInterface.NetworkInterfaceType;
            if (nit == NetworkInterfaceType.None)
            {
                SupportedOrientations = SupportedPageOrientation.Portrait;
                HideSplashImage();
                viewMessage.Visibility = System.Windows.Visibility.Visible;
            }
            else
            {
                //NetworkChange.NetworkAddressChanged += OnNetworkAddressChanged;
                // Start the storyboard to show the stock details
                //RefreshStoryboard();

                // Setup the time which will control the stock details rotation
                _timer = new DispatcherTimer();
                _timer.Interval = new TimeSpan(0, 0, 2);
                _timer.Tick += StockDetailsStoryBoardTimer;
                _timer.Start();

                var timer = new DispatcherTimer();
                timer.Interval = new System.TimeSpan(0, 0, 0, 0, 500);
                timer.Tick += ShowViewTick;
                timer.Start();

                _vm.PropertyChanged += ViewModelPropertyChanged;
            }
        }

        private void PortraitFadeInCompleted(object sender, EventArgs e)
        {
            if (!IsOrientationLandscape(Orientation))
            {
                viewLandScape.Visibility = System.Windows.Visibility.Collapsed;
            }
        }

        private void LandscapeFadeInCompleted(object sender, EventArgs e)
        {
            if (IsOrientationLandscape(Orientation))
            {
                viewPortrait.Visibility = System.Windows.Visibility.Collapsed;
            }
        }

        private void ShowViewTick(object sender, EventArgs e)
        {
            ((DispatcherTimer)sender).Stop();
            if (_showSearchView || _vm.Tickers.Count == 0)
            {
                Dispatcher.BeginInvoke(ShowSearchView);
            }

            Dispatcher.BeginInvoke(HideSplashImage);
        }

        protected override void OnNavigatedFrom(System.Windows.Navigation.NavigationEventArgs e)
        {
            if (_vm != null)
            {
                if (_vm.SelectedStock != null)
                {
                    PhoneApplicationService.Current.State["SelectedStock"] = _vm.SelectedStock.Symbol;
                }

                PhoneApplicationService.Current.State["SearchTerm"] = _vm.StockSearchFilter;
                PhoneApplicationService.Current.State["SearchViewVisible"] = IsSearchViewVisible();
            }
            base.OnNavigatedFrom(e);
        }

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            if (PhoneApplicationService.Current.State.ContainsKey("SelectedStock"))
            {
                _resumeSymbol = PhoneApplicationService.Current.State["SelectedStock"] as string;
            }

            if (PhoneApplicationService.Current.State.ContainsKey("SearchTerm"))
            {
                _searchText = PhoneApplicationService.Current.State["SearchTerm"] as string;
            }

            if (PhoneApplicationService.Current.State.ContainsKey("SearchViewVisible") && ((bool)PhoneApplicationService.Current.State["SearchViewVisible"]))
            {
                _showSearchView = true;
            }

            base.OnNavigatedTo(e);
        }

        /// <summary>
        /// Stocks the details story board timer.
        /// </summary>
        /// <param name="sender">The sender.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        void StockDetailsStoryBoardTimer(object sender, EventArgs e)
        {
            RefreshStoryboard();
        }

        /// <summary>
        /// Handles the OrientationChanged event of the PhoneApplicationPage control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="Microsoft.Phone.Controls.OrientationChangedEventArgs"/> instance containing the event data.</param>
        private void PhoneApplicationPage_OrientationChanged(object sender, OrientationChangedEventArgs e)
        {
            if (IsOrientationLandscape(e.Orientation))
            {
                if (_lastOrientation != PageOrientation.Landscape)
                {
                    viewLandScape.Visibility = System.Windows.Visibility.Visible;
                    viewPortrait.FadeOut.Begin();
                    viewLandScape.FadeIn.Begin();
                }
                _lastOrientation = PageOrientation.Landscape;
            }
            else
            {
                if (_lastOrientation != PageOrientation.Portrait)
                {
                    viewPortrait.Visibility = System.Windows.Visibility.Visible;
                    viewLandScape.FadeOut.Begin();
                    viewPortrait.FadeIn.Begin();
                }
                _lastOrientation = PageOrientation.Portrait;
            }
        }

        /// <summary>
        /// Handles the Up event of the Gesture control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Gesture_Up(object sender, EventArgs e)
        {
            if (_vm.SelectedStock == null) return;
            if (!_vm.RemoveStockCommand.CanExecute(_vm.SelectedStock.Symbol)) return;

            _vm.RemoveStockCommand.Execute(_vm.SelectedStock.Symbol);
        }

        /// <summary>
        /// Handles the Down event of the Gesture control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Gesture_Down(object sender, EventArgs e)
        {
            //viewSearch.Visibility = System.Windows.Visibility.Visible;
            ShowSearchView();
        }

        /// <summary>
        /// Handles the Left event of the Gesture control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Gesture_Left(object sender, EventArgs e)
        {
            // Go forward 1
            _vm.MoveToNextStock.Execute("1");

            //_counter++;

            //ShowHideAbout();
        }

        /// <summary>
        /// Handles the Right event of the Gesture control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void Gesture_Right(object sender, EventArgs e)
        {
            // Go back 1
            _vm.MoveToNextStock.Execute("-1");

            //_counter--;

            //ShowHideAbout();
        }

        private void SearchBarMouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (_vm.HasConnection)
            {
                ShowSearchView();
            }
            else
            {
                ShowNotConnectionMessage();
            }
        }

        private void MainPageBackKeyPress(object sender, CancelEventArgs e)
        {
            if (viewMessage.Visibility == System.Windows.Visibility.Collapsed && IsSearchViewVisible() && _vm != null && _vm.Tickers.Count > 0)
            {
                HideSearchView();
                e.Cancel = true;
            }
        }

        private void ViewModelPropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            if (e.PropertyName == "Tickers" && _vm.Tickers.Count == 0)
            {
                viewSearch.Focus();
                ShowSearchView();
            }
            else if (e.PropertyName == "HasConnection")
            {
                if (_vm.HasConnection)
                {
                    viewPortrait.SearchBarLogo.Opacity = 1;
                    viewLandScape.SearchBarLogo.Opacity = 1;
                }
                else
                {
                    ShowNotConnectionMessage();
                    viewPortrait.SearchBarLogo.Opacity = .2;
                    viewLandScape.SearchBarLogo.Opacity = .2;
                }
            }
        }

        #endregion Event Handlers

        #region Private Methods
        /// <summary>
        /// Shows/hide about screen.
        /// </summary>
        private void ShowHideAbout()
        {
            viewAbout.Visibility = (_counter <= -2) ? System.Windows.Visibility.Visible : System.Windows.Visibility.Collapsed;
        }

        private void ShowNotConnectionMessage()
        {
            //Style style = (Style) Application.Current.Resources["MessageBoxStyle"];
            //XamMessageBox.Show("Warning", "IG Finance is having trouble connecting to the internet to retrieve stock info. Until your phone gets connected again, we can only show you the last info we have for your stocks.", null, VerticalPosition.Center, style, new XamMessageBoxCommand("Ok", () => {}));
            //MessageBox.Show("IG Finance is having trouble connecting to the internet to retrieve stock info. Until your phone gets connected again, we can only show you the last info we have for your stocks.");
            messageWindow.IsOpen = true;
        }
        
        private void ShowSearchView()
        {
            viewSearch.SlideIn.Begin();
        }

        private void HideSearchView()
        {
            viewSearch.SlideOut.Begin();
        }

        private void HideSplashImage()
        {
            SplashImagePortrait.Visibility = Visibility.Collapsed;
            SplashImageLandscape.Visibility = Visibility.Collapsed;
        }

        private bool IsSearchViewVisible()
        {
            return ((CompositeTransform)viewSearch.LayoutRoot.RenderTransform).TranslateY == 0;
        }

        private void RefreshStoryboard()
        {
            if (_vm.Tickers.Count == 0)
            {
                return;
            }

            if (!_vm.SelectedStock.IsDetailsLoaded)
            {
                return;
            }

            PivotItem pi = (PivotItem)viewPortrait.Pivot.ItemContainerGenerator.ContainerFromIndex(viewPortrait.Pivot.SelectedIndex);
            Grid grid = pi.FindVisualChildByName<Grid>("StockRoot");

            if (grid == null)
            {
                return;
            }

            Storyboard sbSubInfoIn = ((Storyboard)grid.Resources["SbSubInfoIn"]);
            Storyboard sbSubInfoOut = ((Storyboard)grid.Resources["sbSubInfoOut"]);

            TextBlock tb = pi.FindVisualChildByName<TextBlock>("txtblkRange");

            if (tb == null)
            {
                return;
            }

            if (_storyBoardFlag || tb.Opacity == 0)
            {
                if (_vm.IsDetailsPopulated)
                {
                    sbSubInfoIn.Begin();
                    _storyBoardFlag = !_storyBoardFlag;
                    _storyBoardCount = 0;
                }
            }
            else
            {
                if (!_vm.HasConnection)
                {
                    return;
                }

                _storyBoardCount++;
                if (_storyBoardCount == 3)
                {
                    sbSubInfoOut.Begin();
                    _storyBoardFlag = !_storyBoardFlag;
                    _storyBoardCount = 0;
                }
            }
        }

        private bool IsOrientationLandscape(PageOrientation orientation)
        {
            return orientation == PageOrientation.Landscape || orientation == PageOrientation.LandscapeLeft ||
                   orientation == PageOrientation.LandscapeRight;
        }

        #endregion Private Methods

        private void commandButton_Click(object sender, RoutedEventArgs e)
        {
            messageWindow.IsOpen = false;
        }
    }
}